SystemOrganization addCategory: #'Zinc-FileSystem'!

Object subclass: #ZnFileSystemUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-FileSystem'!

!ZnFileSystemUtils commentStamp: '<historical>' prior: 0!
I am ZnFileSystemUtils.
I help is abstracting over API differences pre and post the introduction of FileSystem in Pharo 2.0

Part of Zinc HTTP Components.!

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/3/2012 15:07'!
baseNameFor: fileName
	^ fileName asFileReference basename ! !

!ZnFileSystemUtils class methodsFor: 'defaults' stamp: 'SvenVanCaekenberghe 7/3/2012 15:09'!
defaultDirectory
	^ FileSystem disk workingDirectory! !

!ZnFileSystemUtils class methodsFor: 'defaults' stamp: 'SvenVanCaekenberghe 7/3/2012 15:09'!
defaultDirectoryPath
	^ self defaultDirectory fullName! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/3/2012 15:10'!
delete: path
	^ path asFileReference delete! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/2/2012 16:57'!
deleteIfExists: path
	^ (self exists: path)
		ifTrue: [ self delete: path. true ]
		ifFalse: [ false ]! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/3/2012 15:10'!
downloadPathTo: directoryOfFileName for: baseName
	| downloadPath |
	downloadPath := directoryOfFileName asFileReference.
	^ downloadPath exists
		ifTrue: [ (downloadPath / baseName) fullName ]
		ifFalse: [ directoryOfFileName ]
	! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/3/2012 15:10'!
exists: path
	^ path asFileReference exists! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/3/2012 15:11'!
extensionFor: fileName
	^ fileName asFileReference extension! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/3/2012 16:12'!
fileNamed: fileName do: block
	^ fileName asFileReference 
		fileStreamDo: block! !

!ZnFileSystemUtils class methodsFor: 'queries' stamp: 'SvenVanCaekenberghe 7/3/2012 15:12'!
fileSizeFor: fileName
	^ fileName asFileReference entry size! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/3/2012 16:12'!
fileStreamFor: fileName
	^ fileName asFileReference 
		writeStream! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/3/2012 15:14'!
fullNameFor: fileName
	^ fileName asFileReference fullName! !

!ZnFileSystemUtils class methodsFor: 'queries' stamp: 'SvenVanCaekenberghe 7/3/2012 15:15'!
modificationTimeFor: path
	^ path asFileReference entry modificationTime ! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/3/2012 16:12'!
newFileNamed: fileName do: block
	^ fileName asFileReference 
		writeStreamDo: block
		ifPresent: [ self error: fileName asString, ' already exists' ]! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/3/2012 16:13'!
oldFileNamed: fileName do: block
	^ fileName asFileReference 
		readStreamDo: block
		ifAbsent: [ self error: fileName asString, ' does not exists' ]! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/3/2012 16:13'!
oldFileStreamFor: fileName
	^fileName asFileReference 
		readStreamIfAbsent: [ self error: fileName asString, ' does not exists' ]! !

Object subclass: #ZnStaticFileServerDelegate
	instanceVariableNames: 'prefix directory mimeTypeExpirations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-FileSystem'!

!ZnStaticFileServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnStaticFileServerDelegate.
I am a simple proof of concept implementation of a web server serving static files.
I handle urls with an optional prefix as requests for files in a directory.
I serve index.html or index.htm when a directory is requested and these files exist.
I do a redirect when a path that is not does not end with a / refers to directory.
I function as a delegate for ZnServer.

ZnServer startDefaultOn: 1701.
ZnServer default delegate: ((ZnStaticFileServerDelegate new) 
									prefixFromString: 'apple/macosx'; 
									directory: '/Library/WebServer/Documents' asDirectory; 
									yourself).

Part of Zinc HTTP Components.!

!ZnStaticFileServerDelegate class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2012 10:26'!
defaultMimeTypeExpirations
	"Return a default dictionary mapping ZnMimeTypes to their expiration time in seconds.
	Missing entries will have no expiration time."

	| dict |
	dict := Dictionary new.
	dict at: ZnMimeType textCss put: 30 days asSeconds.
	dict at: ZnMimeType textJavascript put: 30 days asSeconds.
	dict at: ZnMimeType textPlain put: 30 days asSeconds.
	dict at: ZnMimeType textHtml put: 30 days asSeconds.
	dict at: ZnMimeType imageGif put: 30 days asSeconds.
	dict at: ZnMimeType imageJpeg put: 30 days asSeconds.
	dict at: ZnMimeType imagePng put: 30 days asSeconds.
	dict at: ZnMimeType applicationJavascript put: 30 days asSeconds.
	^ dict! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/3/2012 16:38'!
actualFilenameFor: uri
	| subElements subDir entry |
	(uri isEmpty and: [ self prefix isEmpty ]) ifTrue: [ ^ self indexFileIn: self directory ].
	(self prefix isEmpty or: [ uri isEmpty not and: [ uri pathSegments beginsWith: self prefix ] ]) ifFalse: [ ^ nil ].
	subElements := (uri pathSegments allButFirst: self prefix size) reject: [ :each | each = #/ ].
	subDir := subElements allButLast inject: self directory into: [ :parent :sub | | file |
		(file := parent / sub) exists
			ifTrue: [ file ]
			ifFalse: [ ^ nil ] ]. 
	subElements isEmpty
		ifTrue: [ entry := subDir entry ] 
		ifFalse: [ | file |
			 (file := subDir / subElements last) exists 
				ifTrue: [ entry := file entry ] 
				ifFalse: [ ^ nil ] ].
	^ entry isDirectory 
		ifTrue: [ self indexFileIn: entry reference ]
		ifFalse: [ entry reference fullName ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:06'!
directory
	"The FileDirectory whose files I am serving"
	
	^ directory ! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:07'!
directory: fileDirectory
	"Set the FileDirectory whose files I should be serving"

	directory := fileDirectory! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2011 12:35'!
directoryRedirectFor: uri
	| directoryUri |
	directoryUri := uri copy addPathSegment: #/.  
	^ ZnResponse redirect: directoryUri ! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/14/2012 10:30'!
expirationDateFor: aMimeType
	| expiration |
	expiration := self mimeTypeExpirations at: aMimeType ifAbsent: [ ^ nil ].
	^ ZnUtils httpDate: (DateAndTime now asUTC + expiration seconds)! !

!ZnStaticFileServerDelegate methodsFor: 'public' stamp: 'PaulDeBruicker 5/12/2012 09:56'!
handleRequest: request
	"Server delegate entry point"

	| actualFilename |
	(#(#GET #HEAD) includes: request method)
		ifFalse: [ ^ ZnResponse methodNotAllowed: request ].
	actualFilename := self actualFilenameFor: request uri.
	^ actualFilename notNil
		ifTrue: [ 
			(self redirectNeededFor: request uri actualFilename: actualFilename)
				ifTrue: [ self directoryRedirectFor: request uri ]
				ifFalse: [ self responseForFile: actualFilename fromRequest: request ] ]
		ifFalse: [ ZnResponse notFound: request uri ]! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/3/2012 14:43'!
indexFileIn: fileDirectory
	#( 'index.html' 'index.htm' ) do: [ :each | | file |
			(file := fileDirectory / each) exists
				ifTrue: [ 
					^ file fullName ] ].
	^ nil
! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/14/2012 09:41'!
maxAgeFor: aMimeType
	| expiration |
	expiration := self mimeTypeExpirations at: aMimeType ifAbsent: [ ^ nil ].
	^ 'max-age=', expiration asString! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2012 09:39'!
mimeTypeExpirations
	"Return a dictionary mapping ZnMimeTypes to their expiration time in seconds.
	Missing entries will have no expiration time."

	^ mimeTypeExpirations ifNil: [ mimeTypeExpirations := Dictionary new ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2012 09:39'!
mimeTypeExpirations: aDictionary
	"Set the mapping of ZnMimeTypes to their expiration time in seconds to aDictionary.
	Missing entries will have no expiration time."

	mimeTypeExpirations := aDictionary! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 12:46'!
prefix
	"The path prefix collection under which I am serving files"
	
	^ prefix ifNil: [ prefix := #() ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:08'!
prefix: orderedCollection
	"Set the path prefix under which I should be serving files to orderedCollection containing path elements"
	
	prefix := orderedCollection! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:09'!
prefixFromString: string
	"Set the path prefix under which I should be serving files to string,
	interpreting each /-separated token as a path element"

	self prefix: (string findTokens: '/')! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2011 13:11'!
redirectNeededFor: uri actualFilename: actualFilename
	uri isDirectory ifTrue: [ ^ false ].  
	^ (actualFilename endsWith: uri lastPathSegment) not! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/3/2012 14:56'!
responseForFile: filename fromRequest: aRequest
	| file entry size time mimeType fileStream entity response modified |
	file := filename asFileReference.
	entry := file entry.
	size := entry size.
	time := entry modificationTime.
	modified := true.
	aRequest headers
		at: 'If-Modified-Since'
		ifPresent: [ modified := time > (ZnUtils parseHttpDate: (aRequest headers at: 'If-Modified-Since')) ].
	modified
		ifTrue: [ 
			mimeType := ZnMimeType forFilenameExtension: file extension.
			fileStream := file readStream.
			mimeType isBinary
				ifTrue: [ fileStream binary ].
			(entity := ZnStreamingEntity type: mimeType length: size) 
				stream: fileStream.
			(response := ZnResponse ok: entity) 
				headers at: 'Modification-Date' put: (ZnUtils httpDate: time).
			(self maxAgeFor: mimeType) 
				ifNotNil: [ :maxAge | response headers at: 'Cache-Control' put: maxAge ].
			(self expirationDateFor: mimeType)
				ifNotNil: [ :expirationDate | response headers at: 'Expires' put: expirationDate ] ]
		ifFalse: [ response := ZnResponse notModified ].
	^ response! !
